
function love.load()
	print("Loaded")
	shape = {}
	polygons = {}
	rgba = function(rr, bb, gg, aa)
		if rr>255 then rr = 255 end --max
		if rr<0 then rr = 0 end --min
		if gg>255 then gg = 255 end --max
		if gg<0 then gg = 0 end --min
		if bb>255 then bb = 255 end --max
		if bb<0 then bb = 0 end
		if aa>255 then aa = 255 end --max
		if aa<0 then aa = 0 end
		return {r = rr, g = gg, b = bb, a = aa}
	end
	rgb = function(rr, bb, gg)
		return rgba(rr, bb, gg, 255) --h4x, must ALWAYS have alpha
	end
	colours = {rgb(255, 0, 0), rgb(0, 255, 0), rgb(0, 0, 255), rgb(255, 0, 255), rgb(125, 125, 125), rgb(255, 255, 0), rgb(255, 255/2, 0), rgb(0, 255/2, 255), rgb(0, 150, 0)}
	shape.create = function(class, pts, colour)
		if string.lower(class) == "triangle" then
			return {
				points = {
					v2(),
					v2(),
					v2()
				},
				col = colour
			}
		end
		if string.lower(class) == "square" then
			return {
				points = {
					v2(-100, -100),
					v2(-100, 100),
					v2(100, 100),
					v2(100, -100)
				},
				col = colour
			}
		end
		if string.lower(class) == "poly" then
			if pts~=nil then
				if type(pts) == "table" then
					if #pts>0 then
						return {
							points = pts,
							col = colour
						}
					end
				end
			end
		end
	end
	local startshape = shape.create("square", nil, colours[math.random(1, #colours)])
	startshape.points[1].x = 150 startshape.points[1].y = 150
	startshape.points[2].x = 150 startshape.points[2].y = 350
	startshape.points[3].x = 350 startshape.points[3].y = 350
	startshape.points[4].x = 350 startshape.points[4].y = 150
	table.insert(polygons, startshape)
	dragging = false
	dragstart = v2()
	dragshape = 0
	dragcurrent = v2()
	cutting = false
	cutstart = v2()
end

function v2(Xx, Yy)
	if Xx==nil then
		return {x = 0, y = 0}
	elseif Xx~=nil and Yy==nil then
		return {x = Xx, y = 0} --nil
	elseif Xx~=nil and Yy~=nil then
		return {x = Xx, y = Yy}
	end
end
--[[
function getdir(a, b)
	x = b.x-a.x
	y = b.y-a.y
	dx = x
	dy = y
	d = math.atan2(dy, dx)-math.pi/2
	return -d
end
function getdist(a, b)
	dx = a.x-b.x
	dy = a.y-b.y
	d = math.sqrt(dx*dx+dy*dy)
	return d
end--]]
function cross(a, b, c, d)
	local denom =	((d.y - c.y)*(b.x - a.x))-((d.x - c.x)*(b.y - a.y))
	local numa  =	((d.x - c.x)*(a.y - c.y))-((d.y - c.y)*(a.x - c.x))
	local numb  =	((b.x - a.x)*(a.y - c.y))-((b.y - a.y)*(a.x - c.x))
	if denom == 0 then
		if numa == 0 and numb == 0 then
			return false, nil
		end
		return false, nil
	end
	local ua = numa/denom
	local ub = numb/denom
	if (ua >= 0.0 and ua <= 1.0 and ub >= 0.0 and ub <= 1.0) then
		local x = a.x + ua*(b.x - a.x);
		local y = a.y + ua*(b.y - a.y);
		return true, v2(x, y)
	end
	return false, nil
end
function printdebug()
	love.graphics.setColor(0, 0, 0, 255)
	love.graphics.print("Polygons: "..#polygons, 10, 15, 0, 1, 1)
	love.graphics.print("Dragging: "..tostring(dragging), 10, 15*2, 0, 1, 1)
	love.graphics.print("Cutting: "..tostring(cutting), 10, 15*3, 0, 1, 1)
	love.graphics.print("Controls: Left Mouse = Drag, Right Mouse = Cut, Space = New, Q = delete", 10, gr.getHeight()-15, 0, 1, 1)
end
function love.draw()
	gr = love.graphics
	local mx = gr.getWidth()/2
	local my = gr.getHeight()/2
	love.graphics.setBackgroundColor(255, 255, 255)
	dragcurrent.x = love.mouse.getX()
	dragcurrent.y = love.mouse.getY()
	dragdiff = v2(dragcurrent.x - dragstart.x, dragcurrent.y - dragstart.y)
	cutdiff = v2(dragcurrent.x - cutstart.x, dragcurrent.y - cutstart.y)
	for i, v in pairs(polygons) do
		local points = v.points
		gr.setPointSize(5)
		if (dragging == true and dragshape == i) then
			gr.setColor(v.col.r, v.col.g,v.col.b, v.col.a/2)
			for pn = 2, #points-1 do
				gr.triangle( "fill", points[1].x+dragdiff.x, points[1].y+dragdiff.y, points[pn].x+dragdiff.x, points[pn].y+dragdiff.y, points[pn+1].x+dragdiff.x, points[pn+1].y+dragdiff.y )
			end
			gr.setLineWidth(1.2)
			love.graphics.setColor(0, 0, 0, 255/1.2)
			for pn = 1, #points do --boarder
				local pn2 = pn+1
				if pn == #points then pn2 = 1 end
				--gr.line(points[1].x+dragdiff.x, points[1].y+dragdiff.y, points[pn2].x+dragdiff.x, points[pn2].y+dragdiff.y) --view triangles
				gr.line(points[pn].x+dragdiff.x, points[pn].y+dragdiff.y, points[pn2].x+dragdiff.x, points[pn2].y+dragdiff.y)
			end
		else
			gr.setColor(v.col.r, v.col.g,v.col.b, v.col.a)
			for pn = 2, #points-1 do
				gr.triangle( "fill", points[1].x, points[1].y, points[pn].x, points[pn].y, points[pn+1].x, points[pn+1].y )
			end
			gr.setLineWidth(1.2)
			gr.setColor(0, 0, 0, 255/1.2)
			for pn = 1, #points do --boarder, yes, its done by triangles, big deal.
				local pn2 = pn+1
				if pn == #points then pn2 = 1 end
				if points[pn]~= nil and points[pn2]~= nil then
					--gr.line(points[1].x, points[1].y, points[pn2].x, points[pn2].y) --view triangles
					gr.line(points[pn].x, points[pn].y, points[pn2].x, points[pn2].y)
				end
			end
		end
	end
	if (dragdiff.x~=0 or dragdiff.y~=0) and dragging == true then -- render drag line
		gr.setLineWidth(2)
		gr.setColor(0, 255, 0, 255/1.5)
		gr.line(dragstart.x, dragstart.y, dragcurrent.x, dragcurrent.y)
		gr.setPointSize(5)
		gr.point(dragstart.x, dragstart.y)
		gr.point(dragcurrent.x, dragcurrent.y)
	end
	if (cutdiff.x~=0 or cutdiff.y~=0) and cutting == true then -- render cut line
		gr.setLineWidth(2)
		gr.setColor(0, 0, 255, 255/1.5)
		gr.line(cutstart.x, cutstart.y, dragcurrent.x, dragcurrent.y)
		gr.setPointSize(5)
		gr.point(cutstart.x, cutstart.y)
		gr.point(dragcurrent.x, dragcurrent.y)
	end
	printdebug()
end
function iseven(n)
	if n/2==math.floor(n/2) then --if odd, will have .5
		return true
	end
	return false
end
function love.mousepressed( x, y, button)
	if button == "l" then
		if dragging == false then
			for i, v in pairs(polygons) do
				local st = v2(x, y)
				local en = v2(x, y+10000)
				local ps = v.points
				local hitcount = 0
				for pn, p in pairs(ps) do
					local pn2 = pn+1
					if pn == #ps then pn2 = 1 end
					local hit, pos = cross(st, en, p, ps[pn2])
					if hit == true then
						hitcount = hitcount+1 --count hit
					end
				end
				if not iseven(hitcount) and hitcount>0 then
					dragstart.x = x
					dragstart.y = y
					dragging = true
					dragshape = i
				end
			end
		end
	elseif button == "r" then
		cutting = true
		cutstart.x = x
		cutstart.y = y
	end
end
function love.mousereleased( x, y, button )
	if button == "l" then
		if dragging == true then
			dragging = false
			local poly = polygons[dragshape]
			local pt = poly.points
			dragcurrent.x = x
			dragcurrent.y = y
			local dragdiff = v2(dragcurrent.x - dragstart.x, dragcurrent.y - dragstart.y)
			for i, v in pairs(pt) do --place polygon in new position
				v.x = v.x+dragdiff.x
				v.y = v.y+dragdiff.y
			end
		end
	elseif button == "r" then
		if cutting == true then
			--print("Cutting!")
			cutting = false
			local newpolys = {} -- put new polygons in here before altering polygon table
			local polystoremove = {} --collect list of polygons to remove
			for pn, v in pairs(polygons) do
				local pts = v.points
				local sha = {} -- first half
				local shb = {} --second half, only intersects each shape twice
				local ia = nil
				local ib = nil
				local iia1 = nil
				local iib1 = nil
				local iia2 = nil
				local iib2 = nil
				for i, v in pairs(pts) do
					local i2 = i+1
					if i2>#pts then
						i2=1
					end
					local hit, pos = cross(v, pts[i2], cutstart, dragcurrent)
					if hit == true then
						if ia == nil then
							ia = pos
							iia1 = i
							iia2 = i2
						elseif ia~= nil and ib == nil then
							ib = pos
							iib1 = i
							iib2 = i2
						end
					end
				end
				if ia~= nil and ib~= nil then
					--SET SHAPE A:
					table.insert(sha, v2(ia.x+0, ia.y+0))
					for ii = iia2, iib1 do
						table.insert(sha, v2(pts[ii].x+0, pts[ii].y+0))
					end
					table.insert(sha, v2(ib.x+0, ib.y+0))
					--SET SHAPE B:
					table.insert(shb, v2(ib.x+0, ib.y+0))
					if iib2==1 then
						for ii = 1, iia1 do
							table.insert(shb, v2(pts[ii].x+0, pts[ii].y+0))
						end
					else
						if iib2<#pts then
							for ii = iib2, #pts do
								table.insert(shb, v2(pts[ii].x+0, pts[ii].y+0))
							end
						elseif iib2==#pts then
							table.insert(shb, v2(pts[#pts].x+0, pts[#pts].y+0))
						end
						for ii = 1, iia1 do
							table.insert(shb, v2(pts[ii].x+0, pts[ii].y+0))
						end
					end
					table.insert(shb, v2(ia.x+0, ia.y+0))
					--now add to list
					table.insert(newpolys, {points = sha, col = v.col})
					table.insert(newpolys, {points = shb, col = v.col}) --colours[math.random(1, #colours)]
					table.insert(polystoremove, pn)
				end
			end
			if #polystoremove>0 then
				for iii = #polystoremove, 1, -1 do --work backwards
					table.remove(polygons, polystoremove[iii], polystoremove[iii])
				end
			end
			if #newpolys>0 then
				for iv, v in pairs(newpolys) do
					table.insert(polygons, v)
				end
			end
			polystoremove = nil
			newpolys = nil
		end
	end
end
function love.keypressed(key)
	if key == " " then --create new shape
		local startshape = shape.create("square", nil, colours[math.random(1, #colours)])
		startshape.points[1].x = 150 startshape.points[1].y = 150
		startshape.points[2].x = 150 startshape.points[2].y = 350
		startshape.points[3].x = 350 startshape.points[3].y = 350
		startshape.points[4].x = 350 startshape.points[4].y = 150
		table.insert(polygons, startshape)
	end
	if key == "q" then
		local polystoremove = {} --collect list of polygons to remove
		x = love.mouse.getX()
		y = love.mouse.getY()
		for i, v in pairs(polygons) do
			local st = v2(x, y)
			local en = v2(x, y+10000)
			local ps = v.points
			local hitcount = 0
			for pn, p in pairs(ps) do
				local pn2 = pn+1
				if pn == #ps then pn2 = 1 end
				local hit, pos = cross(st, en, p, ps[pn2])
				if hit == true then
					hitcount = hitcount+1 --count hit
				end
			end
			if not iseven(hitcount) and hitcount>0 then
				table.insert(polystoremove, i)
			end
		end
		if #polystoremove>0 then
			for iii = #polystoremove, 1, -1 do --work backwards
				table.remove(polygons, polystoremove[iii], polystoremove[iii])
			end
		end
	end
	if key=="escape" then love.event.push('q') end
end

